/* ----------------------------------------------------------------------------
 * Copyright (c) 2017 Semiconductor Components Industries, LLC (d/b/a
 * ON Semiconductor), All Rights Reserved
 *
 * This code is the property of ON Semiconductor and may not be redistributed
 * in any form without prior written permission from ON Semiconductor.
 * The terms of use and warranty for this code are covered by contractual
 * agreements between ON Semiconductor and the licensee.
 *
 * This is Reusable Code.
 *
 * ------------------------------------------------------------------------- */

#include <malloc.h>
#include <string.h>

#include "ioCeltEncode.h"

static OperationParameters parameters;

static short unencodedFrame[160] =
{
    0x0000, 0x0000, 0xffff, 0x0100, 0xfeff, 0x0200, 0xfeff, 0x0200, 0xffff,
    0x0000, 0x0100, 0xfeff, 0x0200, 0xffff, 0x0100, 0xfeff, 0x0300, 0xfeff,
    0x0200, 0xffff, 0x0200, 0x0000, 0x0100, 0x0000, 0x0000, 0xffff, 0x0000,
    0x0000, 0xffff, 0x0200, 0xfeff, 0x0200, 0x0000, 0xfeff, 0x0300, 0xfcff,
    0x0400, 0xfeff, 0x0000, 0x0000, 0x0000, 0x0000, 0x0200, 0xfeff, 0xffff,
    0x0000, 0x0000, 0x0200, 0xffff, 0x0000, 0xffff, 0x0200, 0xfbff, 0x0500,
    0xfdff, 0x0200, 0x0000, 0x0100, 0xfeff, 0x0100, 0x0000, 0xffff, 0x0400,
    0xfdff, 0x0300, 0xfaff, 0x0800, 0xf9ff, 0x0500, 0xfdff, 0x0100, 0xfdff,
    0x0300, 0x0000, 0xffff, 0x0100, 0x0000, 0x0100, 0x0000, 0x0100, 0xffff,
    0xffff, 0x0100, 0xfeff, 0x0200, 0xffff, 0x0000, 0x0000, 0xffff, 0x0000,
    0x0000, 0x0000, 0x0000, 0x0100, 0x0000, 0x0200, 0xfdff, 0x0400, 0xfbff,
    0x0600, 0xfbff, 0x0200, 0xfeff, 0x0200, 0xfdff, 0x0500, 0xfbff, 0x0200,
    0xffff, 0x0200, 0xfdff, 0x0200, 0x0100, 0xfeff, 0x0200, 0xffff, 0x0000,
    0x0000, 0x0000, 0xffff, 0x0300, 0xfbff, 0x0500, 0xfcff, 0x0300, 0xffff,
    0x0100, 0xffff, 0x0000, 0x0000, 0x0000, 0x0000, 0x0100, 0xfdff, 0x0000,
    0x0200, 0xfeff, 0x0100, 0x0000, 0x0000, 0x0000, 0xfdff, 0x0400, 0xfbff,
    0x0700, 0xfbff, 0x0400, 0xfdff, 0x0300, 0xfdff, 0x0000, 0x0200, 0xfeff,
    0x0400, 0xfeff, 0xfeff, 0x0000, 0x0000, 0x0000, 0x0000
};

static uint8_t encodedFrame[40] =
{
    0x10, 0x61, 0x18, 0x7f, 0xb8, 0x42, 0xe3, 0x57, 0x88, 0x6c, 0x3e, 0x01, 0xb8,
    0x19, 0x71, 0x0e, 0xed, 0x8b, 0xb2, 0xe2, 0xf0, 0x05, 0x51, 0xc9, 0xf9,
    0xf1, 0x89, 0xe2, 0x56, 0x6a, 0xc5, 0x15, 0x22, 0x33, 0xe8, 0xf4, 0x97,
    0x00, 0x00, 0x00
};

/* Define function prototypes for the features we will override from the base */
static void celtRead(pSourceSink source, unsigned char *buffer,
                     uint32_t maxsize);

static void celtWrite(pSourceSink sink, unsigned char *buffer, uint32_t size);

/* ----------------------------------------------------------------------------
 * Function     : makeCeltDecodeSourceSink
 * ----------------------------------------------------------------------------
 * Description  : This IO provider allows test data to be input and output
 *                to the codec and is provided for debug purposes. This
 *                method allows the construction of a suitable control block
 *                using malloc.
 * Inputs       : None
 * Outputs      : Pointer to an IO Source/Sink control block which has been
 *                configured to match the test data indicated above.
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
pSourceSink makeCeltDecodeSourceSink(void)
{
    pSourceSink object = (pSourceSink)malloc(sizeof(struct _SourceSink));
    if (object != NULL)
    {
        object->read = &celtRead;
        object->write = &celtWrite;
    }

    parameters.action = DECODE | DECODE_RESET;
    parameters.sampleRate = 16000;
    parameters.frameSize = 160;
    parameters.blockSize = 40;
    parameters.modeAndChannel = 1;

    return object;
}

/* ----------------------------------------------------------------------------
 * Function     : celtRead
 * ----------------------------------------------------------------------------
 * Description  : Implementation of the read method to provide data for test
 * Inputs       : source : pointer to a source/sink control block
 *                buffer : the buffer which will be used to return the test
 *                data
 *                maxsize : The maximum size of the buffer which must not be
 *                exceeded
 * Outputs      : None
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
static void celtRead(pSourceSink source, unsigned char *buffer,
                     uint32_t maxsize)
{
    memcpy(buffer, encodedFrame, parameters.blockSize);
    memcpy(&source->operation, &parameters, sizeof(parameters));
    source->srcUsed = parameters.blockSize;
    parameters.action = DECODE;
}

/* ----------------------------------------------------------------------------
 * Function     : celtWrite
 * ----------------------------------------------------------------------------
 * Description  : Implementation of the write method to verify test results
 * Inputs       : source : pointer to a source/sink control block
 *                buffer : the buffer which has been used to provide the
 *                test results
 *                size : The used size of the buffer
 * Outputs      : None
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
static void celtWrite(pSourceSink sink, unsigned char *buffer, uint32_t size)
{
    /* don't need checks here, using debugger to verify correct behaviour */
}
